****
*  TBWhile.prg
*  Illustration of TBROWSE with While Condition
*  Modified From TBDEMO/DBUEDIT.prgs which are:
*     Copyright (c) 1990 Nantucket Corp.  All rights reserved.
*
*  Note:  compile with /n/w/a

* This Demo shows Names.dbf consisting of Last, First, Addr, City, State, Zip
* with active index on last + first
* It shows Last Name, First Name, City only for those Last Names That
* Begin With Letter That You Input For The cKey GET.

* This program is entered into the public domain by James J. Orlowski, M.D.
* Correction 11/7/90: added goTopBlock() and goBottomBlock() to simplify code
* Correction 12/2/90: removed PUBLICs nTopNo, nBotNo, and TbWhileSet()
*                     simplified by adding TbWhileTop() and TbWhileBot()
*                     Demo section combined aHead() into the array aFields()
#include "inkey.ch"
#include "setcurs.ch"

****
*  main part of tbwhile
*
PROCEDURE TBWhile
   LOCAL aFields := {}, cKey := "O"
   MEMVAR GetList

   USE Names INDEX Names NEW // indexed on Last + First

   * Pass Heading as character and Field as Block including Alias
   * To eliminate the need to use FIELDWBLOCK() function in BrowsWhile()
   AADD(aFields, {"Last Name" , {||Names->Last}  } )
   AADD(aFields, {"First Name", {||Names->First} } )
   AADD(aFields, {"City"      , {||Names->City}  } )

   SETCOLOR("N/BG")
   CLEAR SCREEN
   @ 5,10 SAY "Enter First Letter Of Last Name:" GET cKey PICTURE "!"
   READ
   CLEAR SCREEN

   SEEK cKey
   IF .NOT. FOUND()
      SET COLOR TO
      CLEAR SCREEN
      ? "Sorry, But There Were NO Records To List"
      QUIT
   ENDIF

   * Names->Last = cKey is the Conditional Block passed to this function
   * you can make it as complicated as you want, but you would then
   * have to modify TBWhileSet() to find first and last records
   * matching your key.
   BrowsWhile( aFields, {||Names->Last = cKey}, cKey, ;
             3, 6, MaxRow() - 2, MaxCol() - 6)

   * Note you can use Compound Condition 
   * such as cLast =: "Pierce            " and cFirst =: "Hawkeye  "
   * by changing above block to:
   *    {||Names->Last = cLast .AND. Names->First = cFirst}
   * and setting cKey := cLast + cFirst

   SET COLOR TO
   @ MaxRow(), 0 CLEAR
RETURN
* EOP TBWhile

***
*  BrowsWhile()
*
FUNCTION BrowsWhile(aFields, bWhileCond, cKey, nTop, nLeft, nBottom, nRight)
LOCAL b, column, i
LOCAL cHead, bField
LOCAL cColorSave, nCursSave
LOCAL lMore, nKey

   /* make new browse object */
   b := TBrowseDB(nTop, nLeft, nBottom, nRight)

   /* default heading and column separators */
   b:headSep := ""
   b:colSep  := "  "
   b:footSep := ""

   /* add custom 'TbSkipWhil' (to handle passed condition) */
   b:skipBlock := {|x| TbSkipWhil(x, bWhileCond)}

   /* Set up substitute goto top and goto bottom */
   /* with While's top and bottom records        */
   b:goTopBlock := {|| TbWhileTop(cKey)}
   b:goBottomBlock := {|| TbWhileBot(cKey)}

   /* colors */
   b:colorSpec := "N/W, N/BG, B/W, B/BG, B/W, B/BG, R/W, B/R"

   /* add a column for each field in the current workarea */
   FOR i = 1 TO LEN(aFields)
      cHead  := aFields[i, 1]
      bField := aFields[i, 2]

      /* make the new column */
      column := TBColumnNew( cHead, bField )
      column:defColor := {3, 4}

      b:addColumn(column)
   NEXT

   /* make a window shadow */
   cColorSave := SETCOLOR("N/N")
   @ nTop+1, nLeft+1 CLEAR TO nBottom+1, nRight+1
   SETCOLOR("W/W")
   @ nTop, nLeft CLEAR TO nBottom, nRight
   SETCOLOR(cColorSave)

   nCursSave := SetCursor(0)

   lMore := .t.
   WHILE (lMore)
      /* stabilize the display */
      WHILE ( .NOT. b:stabilize() )
         nKey := INKEY()
         IF ( nKey <> 0 )
            EXIT        /* (abort IF a key is waiting) */
         ENDIF
      ENDDO

      IF ( b:stable )
         /* display is stable */
         IF ( b:hitTop .OR. b:hitBottom )
            Tone(125, 0)
         ENDIF

         /* everything's done; just wait for a key */
         nKey := INKEY(0)
      ENDIF

      /* process key */
      DO CASE
      CASE ( nKey == K_DOWN )
         b:down()

      CASE ( nKey == K_UP )
         b:up()

      CASE ( nKey == K_PGDN )
         b:pageDown()

      CASE ( nKey == K_PGUP )
         b:pageUp()

      CASE ( nKey == K_CTRL_PGUP )
         b:goTop()

      CASE ( nKey == K_CTRL_PGDN )
         b:goBottom()

      CASE ( nKey == K_RIGHT )
         b:right()

      CASE ( nKey == K_LEFT )
         b:left()

      CASE ( nKey == K_HOME )
         b:home()

      CASE ( nKey == K_END )
         b:end()

      CASE ( nKey == K_CTRL_LEFT )
         b:panLeft()

      CASE ( nKey == K_CTRL_RIGHT )
         b:panRight()

      CASE ( nKey == K_CTRL_HOME )
         b:panHome()

      CASE ( nKey == K_CTRL_END )
         b:panEnd()

      CASE ( nKey == K_ESC )
         lMore := .f.

      CASE ( nKey == K_RETURN )
         lMore := .f.
      ENDCASE
   ENDDO  // for WHILE (lmore)

   SetCursor(nCursSave)

RETURN (.t.)

****
*  TbSkipWhil()
*
STATIC FUNCTION TbSkipWhil(n, bWhileCond)
LOCAL i

   i := 0
   IF ( LASTREC() <> 0 )
      IF ( n == 0 )
         SKIP 0

      ELSEIF ( n > 0 .AND. RECNO() <> LASTREC() )
         WHILE ( i < n )
            SKIP 1
            IF ( EOF() .OR. .NOT. Eval(bWhileCond) )
               SKIP -1
               EXIT
            ENDIF
            i++
         ENDDO

      ELSEIF ( n < 0 )
         WHILE ( i > n )
            SKIP -1
            IF ( BOF() )
               EXIT
            ELSEIF .NOT. Eval( (bWhileCond) )
               SKIP
               EXIT
            ENDIF
            i--
         ENDDO
      ENDIF
   ENDIF
RETURN (i)
* EOFcn TbSkipWhil()

STATIC FUNCTION TbWhileTop(cKey)
   SEEK cKey
RETURN NIL

STATIC FUNCTION TbWhileBot(cKey)
   * SeekLast: Finds Last Record For Matching Key
   * Developed By Jon Cole
   * With softseek set on, seek the first record after condition.
   * This is accomplished by incrementing the right most character of the
   * string cKey by one ascii character.  After SEEKing the new string,
   * back up one record to get to the last record which matches cKey.
   SET SOFTSEEK ON
   SEEK LEFT(cKey, LEN(cKey) -1) + CHR( ASC( RIGHT(cKey,1) ) +1)
   SET SOFTSEEK OFF
   SKIP -1
RETURN NIL
* EOFcn SeekLast

